/*******************************************************************************
*   Author       -  Kiraly Tibor                                               *
*                   http://www.tkiraaly.hu                                     *
*   Date         -  2013.01.03.                                                *
*   Chip         -  Atmel ATtiny2313 & HD44780                                 *
*   Compiler     -  avr-gcc (WinAVR )                                          *
*                                                                              *
*   DaniDisplay V1                                                             *
*   A soros porton varja az utasitasokat, belltsa 9600N1                    *   
*   4 utasitast ismer, a tobbire nem reagal, az utasitas vege CR (13).         *
*   C - kepernyo torles                                                        *
*   I - informacio kiiras                                                      *
*   0 - kiiras felso sorba                                                     *
*   1 - kiiras also sorba                                                      *
*   Kiirasi parancsok masodik betuje szamjegy, amely azt mutatja hanyadik      *
*   Betunel kezdodik a kiiras, azutan a kiirando szoveg kovetkezik.            *
*   pl.: i<CR>, c<CR>, 00Kiraly<CR>, 10Tibor<CR>, 182013<CR>                   *
*                                                                              *
********************************************************************************
*   PonyProg Configuration and Security Bits (bepipalva):                      *
*   Klso keramia szuro oszcilator: SUT0                                        *
*                                                                              *
*******************************************************************************/
//
//
//    LCD bekotese:
//
//    14   LCD D7       - AVR PB3 - 15
//    13   LCD D6       - AVR PB2 - 14
//    12   LCD D5       - AVR PB1 - 13
//    11   LCD D4       - AVR PB0 - 12
//    10   LCD D3       - VDD              ( VDD - 2 soros kijelzo, GND - 1 soros)
//     9   LCD D2       - GND
//     8   LCD D1       - GND
//     7   LCD D0       - GND
//     6   LCD E        - AVR PD6 - 11     (1 - change data - 0)
//     5   LCD RW       - GND              (0- write, 1- read)
//     4   LCD RS       - PD5              (0- istruction, 1- data)
//     3   LCD KONTR    - 10 kOhm trim.
//     2   LCD VDD      - +5V
//     1   LCD VSS      - GND


#ifndef F_CPU
#define F_CPU                4000000             // orajel (MHz)
#endif
#define BAUDRATE             9600
#define CMD_LINE_LENGHT      18                  // parancs sor hossza




#include "tkiraaly_attiny2313.h"
#include "tkiraaly_lcddef.h"
#include <util/delay.h>
#include <avr/pgmspace.h>




void lcd_init4( void);                           // LCD inicializalasa 4 bitre
void lcd_putc( UC);                              // betu kiirasa
void lcd_putcmd( UC);                            // parancskod kiadasa
void lcd_yx( UC, UC);                            // kurzor pozicioja 0..3/0..15
void lcd_cls( void);                             // kepernyo torles
void lcd_puts( const char *);                    // string kiirasa
void lcd_defc( const uint8_t *);                 // egyedi betu(k) beallitasa        
void cmd_line( void);                            // parancssor feldolgozo
void lcd_putbyte_bin( UC);                       // 1 byte kiirasa binarisan
void lcd_putc_hex( UC);                          // 1 hexadecimalis szamjegy kiirasa
void lcd_putbyte_hex( UC);                       // 1 byte kiirasa hexadecimalisan




#define LCD_E                6
#define LCD_E_0              BC( PORTD, LCD_E)
#define LCD_E_1              BS( PORTD, LCD_E)

#define LCD_RS               5
#define LCD_RS_UTASITAS      BC( PORTD, LCD_RS)
#define LCD_RS_ADAT          BS( PORTD, LCD_RS)

#define LED                  4
#define LED_BE               BC( PORTD, LED)
#define LED_KI               BS( PORTD, LED)

#define LCD_PORT             PORTB               // also 4 bit
#define LCD_PORT_ENABLE      DDRB= 0B00001111    // also 4 bit kimenet


#define RE_A                 BTC( PIND, 2)
#define RE_B                 BTC( PIND, 3)
#define RE_P                 BTC( PINB, 4)       // gomb lenyomva?




int main( void)
{
   UC a= 0;
   UC c= 100; 
   IT_DISABLE;
   DDRD= 0B01110000;                            // LCD init
   LCD_PORT_ENABLE;
   lcd_init4();
   lcd_cls();
   lcd_yx( 1, 6);
   lcd_puts( PSTR( "Ready"));

/*
   // test1
   for( ;;)
   {
      if (RE_P) LED_BE;
      else LED_KI;
   }

   // test2
   for( ;;)
   {
      lcd_yx( 0, 0);
      lcd_putbyte_bin( PIND);
      lcd_yx( 0, 10);
      if (RE_P) lcd_putc( '1');
      else lcd_putc( '0');
   }
   // <- 11 01 00 10 11
   // -> 11 10 00 01 11
*/
   for( ;;)
   {
      lcd_yx( 0, 0);
      lcd_putbyte_bin( PIND);
      lcd_yx( 0, 10);
      if (RE_P) lcd_putc( '1');
      else lcd_putc( '0');
      if( (a & 0B00001100) != (PIND & 0B00001100)) // valtozas?
      {
         a= ( a >> 2) | (PIND & 0B00001100);
         if ( a == 0B00001101) c++;              // jobbra
         if ( a == 0B00001110) c--;              // balra
         lcd_yx( 0, 12);
         lcd_putbyte_hex( c);
      }
   }
   return 0;
}




void lcd_init4( void)                            // LCD inicializalasa 4 bitre
{
   _delay_ms( 15);
   LCD_RS_UTASITAS;
   LCD_PORT= 0x00000010;                         // 4 bit interface
   LCD_E_1;
   LCD_E_0;
   _delay_ms( 5);
   LCD_E_1;
   LCD_E_0;
   _delay_us( 120);
   lcd_putc( 0B00101000);                        // 4 bit interface, 2 sor, 5x8 pontos betu
   lcd_putc( 0B00101000);                        // 2x kell kiadni, vagy LCD D3-t VDD-re kell kotni
   lcd_putc( 0B00001100);                        // kijelzes be, cursor ki
   lcd_putc( 0B00000110);                        // kiiras jobbra
   LCD_RS_ADAT;
}




void lcd_putc_hex( UC c)                         // 1 hexadecimalis szamjegy kiirasa
{
   c&= 0x0F;
   if( c < 10) c+= '0';
   else        c+= 'A'- 10;
   lcd_putc( c);
}




void lcd_putcmd( UC cmd)                         // parancskod kiadasa
{
   LCD_RS_UTASITAS;
   lcd_putc( cmd);
   LCD_RS_ADAT;
}




void lcd_putc( UC c)                             // egy betu kiiras
{
   LCD_E_1;                                      // felso 4 bit
   LCD_PORT= ( LCD_PORT & 0B11110000) | ( 0B00001111 & c >> 4);
   LCD_E_0;
   LCD_E_1;                                      // also 4 bit
   LCD_PORT= ( LCD_PORT & 0B11110000) | ( 0B00001111 & c);
   LCD_E_0;
   _delay_us( 37);                               // var 37 usec  
}




void lcd_yx( UC sor, UC betu)                    //  kurzor pozicionalasa 0..3/0..15
{
   UC cim= 0x80;                                 // parancs kodja
   if (sor & 0B00000001) cim+= 64;               // 1. es 3. sor
   if (sor & 0B00000010) cim+= 20;               // 2. es 3. sor
   cim+= betu & 0x0F; 
   lcd_putcmd( cim);
}




void lcd_cls( void)                              // kepernyo torles
{
   lcd_putcmd( 0x01);
   _delay_ms( 2);
}




void lcd_defc( const uint8_t *s)                 // egyedi betu(k) beallitasa
{
   register unsigned char c;
   LCD_RS_UTASITAS;
   lcd_putc( ( ( pgm_read_byte( s++) & 0B00000111) << 3) | 0B01000000);   // betu cime + parancs 
   LCD_RS_ADAT;
   while ( ( c= pgm_read_byte( s++)) != 0xFF) lcd_putc( c);
   LCD_RS_UTASITAS;
   LCD_RS_ADAT;
}




void lcd_puts( const char *s)                 // string kiiras
{
   register unsigned char c;
   while ( ( c= pgm_read_byte( s++))) lcd_putc( c);
}




void lcd_putbyte_bin( UC c)                   // 1 byte kiirasa binarisa
{
   UC i= 8;
   while( i--)
   {
      if( c & 0x80) lcd_putc( '1');
      else          lcd_putc( '0');
      c= c << 1;
   }
}




void lcd_putbyte_hex( UC c)                       // 1 byte kiirasa hexadecimalisan
{
   lcd_putc_hex( c >> 4);
   lcd_putc_hex( c);
}

