/*******************************************************************************
*   Author       -  Kiraly Tibor                                               *
*                   http://www.tkiraaly.hu                                     *
*   Date         -  2013.01.24.                                                *
*   Chip         -  Atmel ATtiny2313 & HD44780                                 *
*   Compiler     -  avr-gcc (WinAVR )                                          *
*                                                                              *
*   KT Zar 1                                                                   *
*   Egy rotary encoder-rel allithato/nyithato 6 szamjegyes zar.                *   
*   Egy kapcsoloval/jumperr-ral teheto a zar beallitas modba.                  *
*   Ki/be kapcsolast kovetoen a 6 szamjegy modosithato.                        *
*   Visszakapcsolas, majd ki/be kapcsolas utan a beallitott kod kiprobalhato.  *
*   Kod beadasa utan a N<->I kiirasnal a gomb jobbra forditasaval hagyhato     *
*   jova a kod, gombnyomas/balra tekeres hatasara ujra beallithato.            *
*   Az egyes szamjegyek beallitasa kvazi veletlen szamokrol indulnak, hogy     *
*   hallgatozassal ne lehessen a kodott megfejteni.                            *
*                                                                              *
********************************************************************************
*   PonyProg Configuration and Security Bits (bepipalva):                      *
*   Belso 4MHz-es osszc.:     SUT0, CKSEL3, CKSEL2, CKSEL0                     *
*                                                                              *
*******************************************************************************/
//
//
//    LCD bekotese:
//
//    14   LCD D7       - AVR PB3 - 15
//    13   LCD D6       - AVR PB2 - 14
//    12   LCD D5       - AVR PB1 - 13
//    11   LCD D4       - AVR PB0 - 12
//    10   LCD D3       - VDD              ( VDD - 2 soros kijelzo, GND - 1 soros)
//     9   LCD D2       - GND
//     8   LCD D1       - GND
//     7   LCD D0       - GND
//     6   LCD E        - AVR PD6 - 11     (1 - change data - 0)
//     5   LCD RW       - GND              (0- write, 1- read)
//     4   LCD RS       - PD5              (0- istruction, 1- data)
//     3   LCD KONTR    - 10 kOhm trim.
//     2   LCD VDD      - +5V
//     1   LCD VSS      - GND


#ifndef F_CPU
#define F_CPU                4000000             // orajel (MHz)
#endif
#define BAUDRATE             9600
#define F_IT_T0              125                 // IT T0 surusege




#include "tkiraaly_attiny2313.h"
#include "tkiraaly_lcddef.h"
#include <util/delay.h>
#include <avr/pgmspace.h>




void lcd_init4( void);                           // LCD inicializalasa 4 bitre
void lcd_putc( UC);                              // betu kiirasa
void lcd_putcmd( UC);                            // parancskod kiadasa
void lcd_yx( UC, UC);                            // kurzor pozicioja 0..3/0..15
void lcd_cls( void);                             // kepernyo torles
void lcd_puts( const char *);                    // string kiirasa
void lcd_defc( const uint8_t *);                 // egyedi betu(k) beallitasa        
void cmd_line( void);                            // parancssor feldolgozo
void lcd_putc_hex( UC);                          // 1 hexadecimalis szamjegy kiirasa
UC szamjegy( UC);                                // egy szamjegy beallitasa
UC jelado( void);                                // encoder lekerdezese
void ee_write( UC, UC);                          // EEPROM irasa
UC ee_read( UC);                                 // EEPROM olvasasa
void torol( void);                               // LCD masodik soranak torlese




#define LCD_E                6
#define LCD_E_0              BC( PORTD, LCD_E)
#define LCD_E_1              BS( PORTD, LCD_E)

#define LCD_RS               5
#define LCD_RS_UTASITAS      BC( PORTD, LCD_RS)
#define LCD_RS_ADAT          BS( PORTD, LCD_RS)

#define LED                  4
#define LED_BE               BC( PORTD, LED)
#define LED_KI               BS( PORTD, LED)

#define LCD_PORT             PORTB               // also 4 bit
#define LCD_PORT_ENABLE      DDRB= 0B00001111    // also 4 bit kimenet

#define K_FEL                1
#define K_LE                 2
#define K_GOMB               3

#define ZAR_BEALLITAS        BTC( PIND, 0)
#define EE_TILT              9


                                                 
volatile UC rnd= 0;                              // kvazi veletlen szam
volatile unsigned int varakozas= 0;              // szamlalo varakozashoz




ISR( TIMER0_COMPA_vect)                          // 125Hz megszakitas kezelese
{
   if( varakozas) varakozas--;                   // visszaszamlalas
   rnd++;                                        // kvazi veletlen szam leptetese
}




int main( void)
{

   UC kod[ 6];
   UC i;
   UC allapot;
   LED_KI;

   TCCR0A= T_CTC;                                // T0 init, 125HZ
   TCCR0B= T_CS256;
   OCR0A= F_CPU/ 256/ F_IT_T0;
   TIMSK= IT_T0CPA_ENABLE;
   IT_ENABLE;

   DDRD= 0B01110000;                             // LCD init
   LCD_PORT_ENABLE;
   lcd_init4();
   lcd_cls();
   lcd_yx( 0, 1);
   lcd_puts( PSTR( "KT_ZAR1 - 2013"));

   if( ZAR_BEALLITAS)                            // uj kod bekeres
   {
      lcd_yx( 1, 3);
      lcd_puts( PSTR( "Beallitas"));
      while( jelado() != K_GOMB);                // varakozas gombnyomasra
      do
      {
         torol();
         kod[ 0]= szamjegy( 1);
         kod[ 1]= szamjegy( 2);
         kod[ 2]= szamjegy( 4);
         kod[ 3]= szamjegy( 5);
         kod[ 4]= szamjegy( 7);
         kod[ 5]= szamjegy( 8);
         lcd_yx( 1, 11);
         lcd_puts( PSTR( "N<->I"));
      } while ( K_FEL != jelado());                 // varakozas FEL tekeresre
      for( i= 0; i < 6; i++) ee_write( i, kod[i]);  // rogzites
      torol();
      lcd_yx( 1, 4);
      lcd_puts( PSTR( "Rogzitve"));
      ee_write( EE_TILT, 0);
      for(;;);
   }

   if( ee_read( EE_TILT))                        // ha ujrainditotta, akkor is ki kell varni a tiltast  
   {
      lcd_yx( 1, 6);
      lcd_puts( PSTR( "Tilt"));
      varakozas= 12500;                          // 100 sec varakozas
      while( varakozas);
      ee_write( EE_TILT, 0);
   }

   for(;;)                                       // kod bekerese
   {
      LED_KI;
      torol();
      lcd_yx( 1, 5);
      lcd_puts( PSTR( "Zarva"));
      while( jelado() != K_GOMB);                // varakozas gombnyomasra
      do
      {
         torol();
         kod[ 0]= szamjegy( 1);
         kod[ 1]= szamjegy( 2);
         kod[ 2]= szamjegy( 4);
         kod[ 3]= szamjegy( 5);
         kod[ 4]= szamjegy( 7);
         kod[ 5]= szamjegy( 8);
         lcd_yx( 1, 11);
         lcd_puts( PSTR( "N<->I"));              // varakozas FEL tekeresre
      } while ( K_FEL != jelado());
      torol();

      allapot= 1;                                // kod ellenorzese
      for( i= 0; i < 6; i++)
      {
         if( kod[i] != ee_read( i)) allapot= 0;
      }
      if( allapot) 
      {                                          // jo kod
         LED_BE;
         lcd_yx( 1, 5);
         lcd_puts( PSTR( "Nyitva"));
         varakozas= 600;                         // 4 sec varakozas
         while( varakozas);
      }
      else
      {                                          // rossz kod
         ee_write( EE_TILT, 1);
         lcd_yx( 1, 6);
         lcd_puts( PSTR( "Tilt"));
         varakozas= 12500;                       // 100 sec varakozas
         while( varakozas);
         ee_write( EE_TILT, 0);
      }
   }   
   return 0;
}




void torol( void)                                // LCD masodik soranak torlese
{
   UC i= 16;
   lcd_yx( 1, 0);
   while( i--) lcd_putc( ' ');
}




// <- 11 01 00 10 11     vezetekek allapota egy kattanasnyi elfordulas kozben
// -> 11 10 00 01 11

UC jelado( void)                                 // encoder lekerdezese
{
   static UC e= 0;                               // A-B vezetekek elozo allapotai
   static UC eg= 0;                              // nyomogomb elzo allapota
   UC a= 0;                                      // visszaadott kod
   UC i;                                         // A-B vezetek lekerdezesehez
   while( a == 0)
   {
      if( BTC( PINB, 4))                         // gomb le van nyomva?
      {
         if( eg == 0) a= K_GOMB;                 // ha korabban nem volt
         eg= 1;
      }
      else                                       // gomb nincs lenyomva
      {
         eg= 0;
         i= (PIND & 0B00001100) >> 2;            // A-B vezetekek beolvasasa
         if( (e & 0B00000011) != i)              // valtozas?
         {
            e= ( e << 2) | i;                    // valtozas eltarolasa
            if ( e == 0B10000111) a= K_FEL;      // jobbra
            if ( e == 0B01001011) a= K_LE;       // balra
         }
      }
      _delay_us( 500);                           // perges menetesites
   }
   return a;   
}




void ee_write( UC cim, UC adat)                  // EEPROM irasa
{
   while( BTS( EECR, EEPE));                     // varakozas mig elozo iras befejezodik
   EEAR= cim;
   EEDR= adat;
   BS( EECR, EEMPE);
   BS( EECR, EEPE);
}




UC ee_read( UC cim)                              // EEPROM olvasasa
{
   while( BTS( EECR, EEPE));                     // varakozas mig elozo iras befejezodik
   EEAR= cim;
   BS( EECR, EERE);
   return EEDR;
}




UC szamjegy( UC x)                               // egy szamjegy beallitasa
{
   UC n;                                         // szamjegy
   UC k= 0;
   n= rnd & 0B00000111;                          // kvazi veletlen kezdoertek
   while( k != K_GOMB)                           // amig nem nyomjak meg a gombot
   {
      lcd_yx( 1, x);
      lcd_putc_hex( n);
      k= jelado();                               // varakozas beavatkozasra
      if( k == K_FEL) n++;
      if( k == K_LE) n--;
      if( n == 10) n= 0;
      if( n == 255) n= 9;
   }
   return( n);
}




void lcd_init4( void)                            // LCD inicializalasa 4 bitre
{
   _delay_ms( 15);
   LCD_RS_UTASITAS;
   LCD_PORT= 0x00000010;                         // 4 bit interface
   LCD_E_1;
   LCD_E_0;
   _delay_ms( 5);
   LCD_E_1;
   LCD_E_0;
   _delay_us( 120);
   lcd_putc( 0B00101000);                        // 4 bit interface, 2 sor, 5x8 pontos betu
   lcd_putc( 0B00101000);                        // 2x kell kiadni, vagy LCD D3-t VDD-re kell kotni
   lcd_putc( 0B00001100);                        // kijelzes be, cursor ki
   lcd_putc( 0B00000110);                        // kiiras jobbra
   LCD_RS_ADAT;
}




void lcd_putc_hex( UC c)                         // 1 hexadecimalis szamjegy kiirasa
{
   c&= 0x0F;
   if( c < 10) c+= '0';
   else        c+= 'A'- 10;
   lcd_putc( c);
}




void lcd_putcmd( UC cmd)                         // parancskod kiadasa
{
   LCD_RS_UTASITAS;
   lcd_putc( cmd);
   LCD_RS_ADAT;
}




void lcd_putc( UC c)                             // egy betu kiiras
{
   LCD_E_1;                                      // felso 4 bit
   LCD_PORT= ( LCD_PORT & 0B11110000) | ( 0B00001111 & c >> 4);
   LCD_E_0;
   LCD_E_1;                                      // also 4 bit
   LCD_PORT= ( LCD_PORT & 0B11110000) | ( 0B00001111 & c);
   LCD_E_0;
   _delay_us( 37);                               // var 37 usec  
}




void lcd_yx( UC sor, UC betu)                    //  kurzor pozicionalasa 0..3/0..15
{
   UC cim= 0x80;                                 // parancs kodja
   if (sor & 0B00000001) cim+= 64;               // 1. es 3. sor
   if (sor & 0B00000010) cim+= 20;               // 2. es 3. sor
   cim+= betu & 0x0F; 
   lcd_putcmd( cim);
}




void lcd_cls( void)                              // kepernyo torles
{
   lcd_putcmd( 0x01);
   _delay_ms( 2);
}




void lcd_puts( const char *s)                   // string kiiras
{
   register unsigned char c;
   while ( ( c= pgm_read_byte( s++))) lcd_putc( c);
}
